import { createToken, createPlugin } from 'fusion-core';
import { matchPath } from 'fusion-plugin-react-router';
import bodyparser from 'koa-bodyparser';

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var HTTPRouterToken = createToken('HTTPRouter');
var HTTPHandlersToken = createToken('HTTPHandlersToken');
var BodyParserOptionsToken = createToken('BodyParserOptionsToken');

function getMatchedPath(currentPath, routes) {
  var paths = Object.keys(routes);
  var matchedPath = paths.find(function (path) {
    var _ref = matchPath(currentPath, path) || {},
        isExact = _ref.isExact;

    return isExact;
  });
  return matchedPath;
}

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var ResponseError =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose(ResponseError, _Error);

  function ResponseError(message) {
    var _this;

    _this = _Error.call(this, message) || this;
    _this.code = null;
    _this.meta = null;
    Error.captureStackTrace(_this, ResponseError);
    return _this;
  }

  return ResponseError;
}(Error);

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-env node */
var serverPlugin = false && createPlugin({
  deps: {
    bodyParserOptions: BodyParserOptionsToken.optional,
    handlers: HTTPHandlersToken
  },
  middleware: function middleware(deps) {
    var bodyParserOptions = deps.bodyParserOptions,
        handlers = deps.handlers;
    if (!handlers) throw new Error('Missing handlers registered to HTTPHandlersToken');
    var parseBody = bodyparser(bodyParserOptions);
    return function (ctx, next) {
      return new Promise(function ($return, $error) {
        var matchedPath, handler, _ref, _ref$params, params, error;

        matchedPath = getMatchedPath(ctx.path, handlers);
        handler = handlers[matchedPath] && handlers[matchedPath][ctx.method];

        if (handler) {
          _ref = matchPath(ctx.path, matchedPath) || {}, _ref$params = _ref.params, params = _ref$params === void 0 ? {} : _ref$params;
          return Promise.resolve(parseBody(ctx, function () {
            return Promise.resolve();
          })).then(function ($await_3) {
            try {
              var $Try_1_Post = function () {
                try {
                  return $If_2.call(this);
                } catch ($boundEx) {
                  return $error($boundEx);
                }
              }.bind(this);

              var $Try_1_Catch = function (e) {
                try {
                  error = e instanceof ResponseError ? e : new Error('UnknownError - Use ResponseError from fusion-plugin-http-router package for more detailed error messages');
                  ctx.body = {
                    status: 'failure',
                    data: {
                      message: error.message,
                      // $FlowFixMe
                      code: error.code,
                      // $FlowFixMe
                      meta: error.meta
                    }
                  };
                  return $Try_1_Post();
                } catch ($boundEx) {
                  return $error($boundEx);
                }
              };

              try {
                return Promise.resolve(handler(_objectSpread({}, params, ctx.request.body), ctx)).then(function ($await_4) {
                  try {
                    ctx.body = $await_4;
                    return $Try_1_Post();
                  } catch ($boundEx) {
                    return $Try_1_Catch($boundEx);
                  }
                }, $Try_1_Catch);
              } catch (e) {
                $Try_1_Catch(e);
              }
            } catch ($boundEx) {
              return $error($boundEx);
            }
          }.bind(this), $error);
        }

        function $If_2() {
          return $return(next());
        }

        return $If_2.call(this);
      });
    };
  }
});

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
// import browserDataFetching from './browser'
var index = false && serverPlugin;

export default index;
export { ResponseError, BodyParserOptionsToken, HTTPHandlersToken, HTTPRouterToken };
//# sourceMappingURL=browser.es5.es.js.map
