'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fusionCore = require('fusion-core');
var fusionPluginReactRouter = require('fusion-plugin-react-router');
var bodyparser = _interopDefault(require('koa-bodyparser'));

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
const HTTPRouterToken = fusionCore.createToken('HTTPRouter');
const HTTPHandlersToken = fusionCore.createToken('HTTPHandlersToken');
const BodyParserOptionsToken = fusionCore.createToken('BodyParserOptionsToken');

function getMatchedPath(currentPath, routes) {
  const paths = Object.keys(routes);
  const matchedPath = paths.find(path => {
    const {
      isExact
    } = fusionPluginReactRouter.matchPath(currentPath, path) || {};
    return isExact;
  });
  return matchedPath;
}

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
class ResponseError extends Error {
  constructor(message) {
    super(message);
    this.code = null;
    this.meta = null;
    Error.captureStackTrace(this, ResponseError);
  }

}

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-env node */
var serverPlugin = true && fusionCore.createPlugin({
  deps: {
    bodyParserOptions: BodyParserOptionsToken.optional,
    handlers: HTTPHandlersToken
  },
  middleware: deps => {
    const {
      bodyParserOptions,
      handlers
    } = deps;
    if (!handlers) throw new Error('Missing handlers registered to HTTPHandlersToken');
    const parseBody = bodyparser(bodyParserOptions);
    return async (ctx, next) => {
      const matchedPath = getMatchedPath(ctx.path, handlers);
      const handler = handlers[matchedPath] && handlers[matchedPath][ctx.method];

      if (handler) {
        const {
          params = {}
        } = fusionPluginReactRouter.matchPath(ctx.path, matchedPath) || {};
        await parseBody(ctx, () => Promise.resolve());

        try {
          ctx.body = await handler(_objectSpread({}, params, ctx.request.body), ctx);
        } catch (e) {
          const error = e instanceof ResponseError ? e : new Error('UnknownError - Use ResponseError from fusion-plugin-http-router package for more detailed error messages');
          ctx.body = {
            status: 'failure',
            data: {
              message: error.message,
              // $FlowFixMe
              code: error.code,
              // $FlowFixMe
              meta: error.meta
            }
          };
        }
      }

      return next();
    };
  }
});

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
// import browserDataFetching from './browser'
var index = true && serverPlugin;

exports.default = index;
exports.ResponseError = ResponseError;
exports.BodyParserOptionsToken = BodyParserOptionsToken;
exports.HTTPHandlersToken = HTTPHandlersToken;
exports.HTTPRouterToken = HTTPRouterToken;
//# sourceMappingURL=index.js.map
